using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using FooCfgToXml;
using System.IO;

namespace UiColumnsInterpreters.ItemInterpreters
{
	public abstract class SplitterItemInterpreter : IItemInterpreter
	{
		public abstract Guid TypeGuid { get; }
		public abstract string TypeName { get; }
		
		public void WriteValueToXml(XmlWriter writer, byte[] itemSettings, Dictionary<Guid, IItemInterpreter> itemInterpreters)
		{
			if (itemSettings.Length == 0)
				return;

			MemoryStream reader = new MemoryStream(itemSettings, false);

			writer.WriteStartElement("Splitter");
			writer.WriteAttributeString("flag", XmlConvert.ToString(CfgToXmlUtilities.ReadInt(reader))); //This always appears to be 0. Don't know what its for, so give it a generic name
			int numberOfChildren = CfgToXmlUtilities.ReadInt(reader);

			for (int i = 0; i < numberOfChildren; i++)
			{
				writer.WriteStartElement("Item");

				IItemInterpreter itemInterpreter = Utilities.WriteItemTypeToXml(writer, reader, itemInterpreters);

				writer.WriteAttributeString("captionOrientation", ReadOrientation(reader));
				writer.WriteAttributeString("locked", XmlConvert.ToString(CfgToXmlUtilities.ReadBool(reader)));
				writer.WriteAttributeString("hidden", XmlConvert.ToString(CfgToXmlUtilities.ReadBool(reader)));
				writer.WriteAttributeString("showCaption", XmlConvert.ToString(CfgToXmlUtilities.ReadBool(reader)));
				writer.WriteAttributeString("autoHide", XmlConvert.ToString(CfgToXmlUtilities.ReadBool(reader)));
				writer.WriteAttributeString("size", XmlConvert.ToString(CfgToXmlUtilities.ReadInt(reader)));
				writer.WriteAttributeString("createLeftToggleArea", XmlConvert.ToString(CfgToXmlUtilities.ReadBool(reader)));

				Utilities.WriteItemSettingsToXml(writer, reader, itemInterpreter, itemInterpreters);
			
				writer.WriteStartElement("CustomTitle");
				writer.WriteAttributeString("enabled", XmlConvert.ToString(CfgToXmlUtilities.ReadBool(reader)));
				writer.WriteString(CfgToXmlUtilities.ReadString(reader, false));
				writer.WriteEndElement(); //CustomTitle

				writer.WriteEndElement(); //Item
			}
			writer.WriteEndElement(); //Splitter
		}

		public void WriteXmlToValue(Stream writer, XmlElement itemSettings, Dictionary<string, IItemInterpreter> itemInterpreters)
		{
			if (itemSettings.ChildNodes.Count == 0)
				return;

			XmlElement splitter = (XmlElement)itemSettings.FirstChild; //Should be first and only child
			XmlToCfgUtilities.WriteInt(writer, XmlConvert.ToInt32(splitter.GetAttribute("flag")));

			XmlNodeList childItemElements = splitter.SelectNodes("Item");
			XmlToCfgUtilities.WriteInt(writer, childItemElements.Count);
			foreach (XmlElement childItem in childItemElements)
			{
				IItemInterpreter itemInterpreter = Utilities.WriteItemTypeToCfg(writer, childItem, itemInterpreters);

				WriteOrientation(writer, childItem.GetAttribute("captionOrientation"));
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(childItem.GetAttribute("locked")));
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(childItem.GetAttribute("hidden")));
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(childItem.GetAttribute("showCaption")));
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(childItem.GetAttribute("autoHide")));
				XmlToCfgUtilities.WriteInt(writer, XmlConvert.ToInt32(childItem.GetAttribute("size")));
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(childItem.GetAttribute("createLeftToggleArea")));

				Utilities.WriteItemSettingsToCfg(writer, childItem, itemInterpreter, itemInterpreters);

				XmlElement customTitle = (XmlElement)childItem.SelectSingleNode("CustomTitle");
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(customTitle.GetAttribute("enabled")));
				XmlToCfgUtilities.WriteString(writer, customTitle.InnerText, true, false);
			}
		}

		private string ReadOrientation(MemoryStream reader)
		{
			int orientation = CfgToXmlUtilities.ReadInt(reader);
			switch (orientation)
			{
				case 0:
					return "horizontal";
				case 1:
					return "vertical";
			}
			//Unlikely to be other orientations, but just in case
			return XmlConvert.ToString(orientation);
		}
		private void WriteOrientation(Stream writer, string orientation)
		{
			int value;
			switch (orientation)
			{
				case "horizontal":
					value = 0;
					break;
				case "vertical":
					value = 1;
					break;
				default:
					//Unlikely to be other orientations, but just in case
					value = XmlConvert.ToInt32(orientation);
					break;
			}

			XmlToCfgUtilities.WriteInt(writer, value);
		}
	}

	public class HorizontalSplitterItemInterpreter : SplitterItemInterpreter
	{
		private static readonly Guid sGuid = new Guid("8fa0bc24-882a-4fff-8a3b-215ea7fbd07f");
		
		public override Guid TypeGuid
		{
			get { return sGuid; }
		}
		public override string TypeName
		{
			get { return "Horizontal splitter"; }
		}
	}

	public class VerticalSplitterItemInterpreter : SplitterItemInterpreter
	{
		private static readonly Guid sGuid = new Guid("77653a44-66d1-49e0-9a7a-1c71898c0441");

		public override Guid TypeGuid
		{
			get { return sGuid; }
		}
		public override string TypeName
		{
			get { return "Vertical splitter"; }
		}
	}
}
