using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using FooCfgToXml;

namespace UiColumnsInterpreters.ItemInterpreters
{
	public class TabedPanelItemInterpreter : IItemInterpreter
	{
		private static readonly Guid sGuid = new Guid("e3956ea0-2bc8-4edf-914f-1209943bdd51");

		public Guid TypeGuid
		{
			get { return sGuid; }
		}
		public string TypeName
		{
			get { return "Tabbed panel stack mod"; }
		}

		public void WriteValueToXml(XmlWriter writer, byte[] itemSettings, Dictionary<Guid, IItemInterpreter> itemInterpreters)
		{
			if (itemSettings.Length == 0)
				return;

			MemoryStream reader = new MemoryStream(itemSettings, false);

			writer.WriteStartElement("TabbedPanel");
			
			Assert(CfgToXmlUtilities.ReadString(reader, false), "tab_orientation");
			writer.WriteAttributeString("orientation", ReadOrientation(reader));
			
			Assert(CfgToXmlUtilities.ReadString(reader, false), "selected_tab");
			writer.WriteAttributeString("selectedTab", XmlConvert.ToString(CfgToXmlUtilities.ReadInt(reader)));
			
			Assert(CfgToXmlUtilities.ReadString(reader, false), "hide_tab");
			writer.WriteAttributeString("hideTabBar", XmlConvert.ToString(CfgToXmlUtilities.ReadInt(reader) == 1)); //Int is used to store bool value
			
			Assert(CfgToXmlUtilities.ReadString(reader, false), "limit_min_height");
			writer.WriteAttributeString("limitMinHeight", XmlConvert.ToString(CfgToXmlUtilities.ReadInt(reader)));
			
			Assert(CfgToXmlUtilities.ReadString(reader, false), "ext_data");
			int numberOfChildren = CfgToXmlUtilities.ReadInt(reader);
			for (int i = 0; i < numberOfChildren; i++)
			{
				writer.WriteStartElement("Item");

				IItemInterpreter itemInterpreter = Utilities.WriteItemTypeToXml(writer, reader, itemInterpreters);
				
				writer.WriteAttributeString("tabCaption", CfgToXmlUtilities.ReadString(reader, false));

				Utilities.WriteItemSettingsToXml(writer, reader, itemInterpreter, itemInterpreters);
				
				writer.WriteEndElement(); //Item				
			}
			writer.WriteEndElement(); //TabbedPanel
		}

		private void Assert(string readString, string expectedString)
		{
			if (readString != expectedString)
			{
				Console.WriteLine("Unexpected tabbed panel stack mod setting name ({0})! Check results carefully", readString);
			}
		}

		public void WriteXmlToValue(Stream writer, XmlElement itemSettings, Dictionary<string, IItemInterpreter> itemInterpreters)
		{
			if (itemSettings.ChildNodes.Count == 0)
				return;

			XmlElement tabbedPanel = (XmlElement)itemSettings.FirstChild; //Should be first and only child

			XmlToCfgUtilities.WriteString(writer, "tab_orientation", true, false);
			WriteOrientation(writer, tabbedPanel.GetAttribute("orientation"));

			XmlToCfgUtilities.WriteString(writer, "selected_tab", true, false);
			XmlToCfgUtilities.WriteInt(writer, XmlConvert.ToInt32(tabbedPanel.GetAttribute("selectedTab")));

			XmlToCfgUtilities.WriteString(writer, "hide_tab", true, false);
			XmlToCfgUtilities.WriteInt(writer, XmlConvert.ToBoolean(tabbedPanel.GetAttribute("hideTabBar")) ? 1 : 0 ); //Bool stored as Int in config

			XmlToCfgUtilities.WriteString(writer, "limit_min_height", true, false);
			XmlToCfgUtilities.WriteInt(writer, XmlConvert.ToInt32(tabbedPanel.GetAttribute("limitMinHeight")));

			XmlToCfgUtilities.WriteString(writer, "ext_data", true, false);

			XmlNodeList childItemElements = tabbedPanel.SelectNodes("Item");
			XmlToCfgUtilities.WriteInt(writer, childItemElements.Count);
			foreach (XmlElement childItem in childItemElements)
			{
				IItemInterpreter itemInterpreter = Utilities.WriteItemTypeToCfg(writer, childItem, itemInterpreters);

				XmlToCfgUtilities.WriteString(writer, childItem.GetAttribute("tabCaption"), true, false);

				Utilities.WriteItemSettingsToCfg(writer, childItem, itemInterpreter, itemInterpreters);
			}
		}

		private string ReadOrientation(MemoryStream reader)
		{
			int orientation = CfgToXmlUtilities.ReadInt(reader);
			switch (orientation)
			{
				case 0:
					return "top";
				case 1:
					return "bottom";
			}
			//Unlikely to be other orientations, but just in case
			return XmlConvert.ToString(orientation);
		}
		private void WriteOrientation(Stream writer, string orientation)
		{
			int value;
			switch (orientation)
			{
				case "top":
					value = 0;
					break;
				case "bottom":
					value = 1;
					break;
				default:
					//Unlikely to be other orientations, but just in case
					value = XmlConvert.ToInt32(orientation);
					break;
			}

			XmlToCfgUtilities.WriteInt(writer, value);
		}
	}
}
