using System;
using System.Collections.Generic;
using System.Text;
using FooCfgToXml;
using System.IO;
using System.Xml;

namespace UiColumnsInterpreters
{
	public class ToolbarsInterpreter : IValueInterpreter
	{
		private static readonly Guid sSettingGuid = new Guid("d26d3aa5-9157-bf8e-d59f-44861c7a82c7");
		
		public Guid SettingGuid
		{
			get { return sSettingGuid; }
		}

		public void WriteValueToXml(XmlWriter writer, byte[] value)
		{
			//The toolbars use the same items as when in layouts, so use the same item interpreters for them.
			Dictionary<Guid, IItemInterpreter> itemInterpreters = FooCfgToXml.Utilities.LoadAddins<Guid, IItemInterpreter>(delegate(IItemInterpreter itemInterpreter) { return itemInterpreter.TypeGuid; });

			MemoryStream reader = new MemoryStream(value, false);

			writer.WriteStartElement("Toolbars");
			int numberOfToolbars = CfgToXmlUtilities.ReadInt(reader);

			writer.WriteStartElement("Row");
			for (int i = 0; i < numberOfToolbars; i++)
			{
				Guid itemType = CfgToXmlUtilities.ReadGuid(reader);
				int width = CfgToXmlUtilities.ReadInt(reader);
				if (CfgToXmlUtilities.ReadBool(reader)) //StartsRow
				{
					writer.WriteEndElement(); //Row
					writer.WriteStartElement("Row");
				}

				writer.WriteStartElement("Toolbar");
				writer.WriteAttributeString("width", XmlConvert.ToString(width));
				
				IItemInterpreter itemInterpreter = Utilities.WriteItemTypeToXml(writer, itemType, itemInterpreters);
				
				Utilities.WriteItemSettingsToXml(writer, reader, itemInterpreter, itemInterpreters);
				writer.WriteEndElement(); //Toolbar
			}
			writer.WriteEndElement(); //Row

			writer.WriteEndElement(); //Toolbars
		}
		public void WriteXmlToValue(Stream writer, XmlElement value)
		{
			//The toolbars use the same items as when in layouts, so use the same item interpreters for them.
			Dictionary<string, IItemInterpreter> itemInterpreters = FooCfgToXml.Utilities.LoadAddins<string, IItemInterpreter>(delegate(IItemInterpreter itemInterpreter) { return itemInterpreter.TypeName; });

			XmlElement toolbars = (XmlElement)value.FirstChild; //Should be first and only child

			XmlNodeList toolbarElements = toolbars.SelectNodes("Row/Toolbar");
			XmlToCfgUtilities.WriteInt(writer, toolbarElements.Count);
			bool startsRow = false; //Even though the first toolbar technically starts a row, it doesn't get the row start flag.
			foreach (XmlElement toolbar in toolbarElements)
			{
				IItemInterpreter itemInterpreter = Utilities.WriteItemTypeToCfg(writer, toolbar, itemInterpreters);
				XmlToCfgUtilities.WriteInt(writer, XmlConvert.ToInt32(toolbar.GetAttribute("width")));
				XmlToCfgUtilities.WriteBool(writer, startsRow);

				Utilities.WriteItemSettingsToCfg(writer, toolbar, itemInterpreter, itemInterpreters);

				if (toolbar.NextSibling == null) //No more toolbars on this row
					startsRow = true; //Next toolbar is the row-starter
				else
					startsRow = false;
			}
		}
	}
}
