using System;
using System.Collections.Generic;
using System.Text;
using FooCfgToXml;
using System.Windows.Forms;
using System.Xml;
using System.IO;

namespace CoreInterpreters
{
	public class KeyboardShortcutsInterpreter : IValueInterpreter
	{
		private static readonly Guid sSettingGuid = new Guid("17de00e7-0565-4e6d-a3fc-1d24a327d11d");
		
		[Flags]
		private enum KeyModifiers
		{
			Shift = 0x1,
			Ctrl = 0x2,
			Alt = 0x4,
		}
		private static string[] sTypes = new string[] {"main","context","context / playlist","context / now playing"};

		public Guid SettingGuid
		{
			get { return sSettingGuid; }
		}

		public void WriteValueToXml(XmlWriter writer, byte[] value)
		{
			MemoryStream reader = new MemoryStream(value, false);
			writer.WriteStartElement("KeyboardShortcuts");

			while (reader.Position < reader.Length)
			{
				writer.WriteStartElement("KeyboardShortcut");

				int keyCode = reader.ReadByte();
				writer.WriteAttributeString("key", ((Keys)keyCode).ToString());

				int modifier = reader.ReadByte();
				writer.WriteAttributeString("modifier", ((KeyModifiers)modifier).ToString());

				writer.WriteAttributeString("global", XmlConvert.ToString(CfgToXmlUtilities.ReadBool(reader)));
				writer.WriteAttributeString("flag", XmlConvert.ToString((byte)reader.ReadByte())); //Don't know what this byte is for

				//Read now, to write later
				byte[] commandIdentifierBytes = new byte[32];
				reader.Read(commandIdentifierBytes, 0, commandIdentifierBytes.Length);

				writer.WriteAttributeString("type", CfgToXmlUtilities.ReadEnum(reader, sTypes));

				writer.WriteStartElement("CommandIdentifier");
				CfgToXmlUtilities.WriteByteArrayToXml(writer, commandIdentifierBytes);
				writer.WriteEndElement(); //CommandIdentifer
				writer.WriteEndElement(); //KeyboardShortcut
			}

			writer.WriteEndElement(); //KeyboardShortcuts
		}

		public void WriteXmlToValue(Stream writer, XmlElement value)
		{
			XmlElement keyboardShortcuts = (XmlElement)value.FirstChild; //Should be first and only child

			XmlNodeList keyboardShortcutElements = keyboardShortcuts.SelectNodes("KeyboardShortcut");
			foreach (XmlElement keyboardShortcutElement in keyboardShortcutElements)
			{
				byte keyCode = (byte)(int)Enum.Parse(typeof(Keys), keyboardShortcutElement.GetAttribute("key"));
				writer.WriteByte(keyCode);
				byte modifier = (byte)(int)Enum.Parse(typeof(KeyModifiers), keyboardShortcutElement.GetAttribute("modifier"));
				writer.WriteByte(modifier);
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(keyboardShortcutElement.GetAttribute("global")));
				writer.WriteByte(XmlConvert.ToByte(keyboardShortcutElement.GetAttribute("flag")));
				XmlToCfgUtilities.WriteXmlToByteArray(writer, (XmlElement)keyboardShortcutElement.SelectSingleNode("CommandIdentifier"));
				XmlToCfgUtilities.WriteEnum(writer, keyboardShortcutElement.GetAttribute("type"), sTypes);
			}
		}
	}
}
