using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace FilterCfgXml
{
	partial class Program
	{
		static void Main(string[] args)
		{
			DisplayTitle();
			if (args.Length == 0)
				DisplayHelp();
			else if (args.Length == 1)
			{
				Console.WriteLine("No components specified");
				DisplayHelp();
			}
			else
			{
				try
				{
					FileInfo fileInfo = new FileInfo(args[0]);
					if (!fileInfo.Exists)
					{
						Console.WriteLine("File does not exist: \"{0}\"", args[0]);
					}
					else
					{
						//Get the components list
						List<string> componentNames = new List<string>(args);
						componentNames.RemoveAt(0); //The first arg is the config filename, so remove it
						
						FilterCfgXml(fileInfo, componentNames);
					}
				}
				catch (Exception e)
				{
					Console.WriteLine("Could not read \"{0}\":\n {1}", args[0], e.Message);
				}
			}
#if DEBUG
			Console.ReadLine();
#endif
		}

		private static void DisplayTitle()
		{
			String title = String.Format("foobar2000 XML config file filter {0}", System.Reflection.Assembly.GetEntryAssembly().GetName().Version);
			Console.WriteLine();
			Console.WriteLine(title);
			Console.WriteLine(new String('=', title.Length));
			Console.WriteLine();
		}

		private static void DisplayHelp()
		{
			Console.WriteLine();
			Console.WriteLine("Usage: FilterCfgXml.exe foobar2000.cfg.xml [\"component\" \"component\"...]");
			Console.WriteLine();
			Console.WriteLine("Examples:");
			Console.WriteLine();
			Console.WriteLine(@"FilterCfgXml.exe ""C:\Program Files\foobar2000\foobar2000.cfg.xml"" foo_ui_columns.dll foo_uie_tabs.dll foo_uie_playlists_dropdown.dll");
			Console.WriteLine();
			Console.WriteLine("  Removes all component settings (including Core settings; add Core to the");
			Console.WriteLine("  argument list to keep it) from the file foobar2000.cfg.xml, except for the");
			Console.WriteLine("  settings for the components foo_ui_columns, foo_uie_tabs, and");
			Console.WriteLine("  foo_uie_playlists_dropdown");
			Console.WriteLine();
		}
	}
}
