using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace FooCfgToXml
{
	partial class Program
	{
		private static void PatchFoobar(FileInfo fileInfo)
		{
			Console.WriteLine("Patching \"{0}\":", fileInfo.FullName);

			//Perorm a few checks to make sure that this is the right foobar exe
			int offset;
			if (fileInfo.Length == 0xEC600)
			{
				Console.WriteLine(" version 0.9.4 detected");
				offset = 0xC4FC;
			}
			else if (fileInfo.Length == 0xEC200)
			{
				Console.WriteLine(" version 0.9.4.1 detected");
				offset = 0xC43A;
			}
			else if (fileInfo.Length == 0xEC000)
			{
				//Both 0.9.4.2 and 0.9.4.3 are this long.
				try
				{
					using (FileStream stream = fileInfo.OpenRead())
					{
						//This byte known to be different
						stream.Seek(60, SeekOrigin.Begin);
						int test = stream.ReadByte();
						if (test == 0x08)
						{
							Console.WriteLine(" version 0.9.4.2 detected");
							offset = 0xC3B2;
						}
						else if (test == 0x00)
						{
							Console.WriteLine(" version 0.9.4.3 detected");
							offset = 0xC4E5;
						}
						else
						{
							Console.WriteLine("Patching aborted: unknown file version.");
							Console.WriteLine("Patching can only be performed on known versions of foobar2000.exe");
							return;
						}
					}
				}
				catch (Exception e)
				{
					Console.WriteLine("Could not read \"{0}\":\n{1}", fileInfo.Name, e.Message);
					return;
				}
			}
			else
			{
				Console.WriteLine("Patching aborted: file size does not match.");
				Console.WriteLine("Patching can only be performed on known versions of foobar2000.exe");
				return;
			}
			try
			{
				using (FileStream stream = fileInfo.OpenRead())
				{
					stream.Seek(offset, SeekOrigin.Begin);
					int test = stream.ReadByte();
					if (test == 0xEB)
					{
						Console.WriteLine("Patching aborted: file is already patched");
						return;
					}
					else if (test != 0x74)
					{
						Console.WriteLine("Patching aborted: file content does not match.");
						Console.WriteLine("Patching can only be performed on a known version of foobar2000.exe");
						return;
					}
				}
			}
			catch (Exception e)
			{
				Console.WriteLine("Could not read \"{0}\":\n{1}", fileInfo.Name, e.Message);
				return;
			}

			FileInfo backupFile = GetNonExistantFilename(new FileInfo(String.Format("{0}.bak", fileInfo.FullName)));
			Console.WriteLine(" Creating backup as: \"{0}\"", backupFile.Name);
			try
			{
				fileInfo.CopyTo(backupFile.FullName);
			}
			catch (Exception e)
			{
				Console.WriteLine("Patching aborted: could not create backup file \"{0}\":\n{1}", backupFile.FullName, e.Message);
				return;
			}

			try
			{
				using (FileStream stream = fileInfo.OpenWrite())
				{
					stream.Seek(offset, SeekOrigin.Begin);
					stream.WriteByte(0xEB);
				}
			}
			catch (Exception e)
			{
				Console.WriteLine("Could not write to \"{0}\":\n{1}", fileInfo.Name, e.Message);
				return;
			}
			Console.WriteLine(" Successfully patched!");
		}
	}
}
