using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;

namespace FooCfgToXml
{
	partial class Program
	{
		private static readonly Guid sFooConfigGuid = new Guid("{5C83AE1C-6D42-43EA-8405-879C254B052C}");

		static void Main(string[] args)
		{
			DisplayTitle();
			if (args.Length == 0)
				DisplayHelp();
			else if (args.Length > 1)
			{
				Console.WriteLine("Too many arguments ({0})", String.Join(" ", args));
				DisplayHelp();
			}
			else
			{
				try
				{
					FileInfo fileInfo = new FileInfo(args[0]);
					if (!fileInfo.Exists)
					{
						Console.WriteLine("File does not exist: \"{0}\"", args[0]);
					}
					else if (fileInfo.Extension.Equals(".exe", StringComparison.InvariantCultureIgnoreCase))
					{
						PatchFoobar(fileInfo);
					}
					else if (fileInfo.Extension.Equals(".cfg", StringComparison.InvariantCultureIgnoreCase))
					{
						ConvertCfgToXml(fileInfo);
					}
					else if (fileInfo.Extension.Equals(".xml", StringComparison.InvariantCultureIgnoreCase))
					{
						ConvertXmlToCfg(fileInfo);
					}
					else
					{
						Console.WriteLine("Unexpected file extension: \"{0}\"", fileInfo.Extension);
						DisplayHelp();
					}
				}
				catch (Exception e)
				{
					Console.WriteLine("Could not read \"{0}\":\n {1}", args[0], e.Message);
				}
			}
#if DEBUG
			Console.ReadLine();
#endif
		}

		private static void DisplayTitle()
		{
			String title = String.Format("foobar2000 config file to XML converter {0}", System.Reflection.Assembly.GetEntryAssembly().GetName().Version);
			Console.WriteLine();
			Console.WriteLine(title);
			Console.WriteLine(new String('=', title.Length));
			Console.WriteLine();
		}

		private static void DisplayHelp()
		{
			Console.WriteLine();
			Console.WriteLine("Usage: FooCfgToXml.exe (foobar2000.cfg|foobar2000.cfg.xml|foobar2000.exe)");
			Console.WriteLine();
			Console.WriteLine("Examples:");
			Console.WriteLine();
			Console.WriteLine(@"FooCfgToXml.exe ""C:\Program Files\foobar2000\foobar2000.cfg""");
			Console.WriteLine("  Reads the foobar2000.cfg file and creates (overwriting if already present)");
			Console.WriteLine("  the file foobar2000.cfg.xml");
			Console.WriteLine();
			Console.WriteLine(@"FooCfgToXml.exe ""C:\Program Files\foobar2000\foobar2000.cfg.xml""");
			Console.WriteLine("  Reads the foobar2000.cfg.xml file and creates (overwriting if already");
			Console.WriteLine("  present) the file foobar2000.cfg. Any existing foobar2000.cfg file");
			Console.WriteLine("  will be backed up to foobar2000.cfg.bak");
			Console.WriteLine();
			Console.WriteLine(@"FooCfgToXml.exe ""C:\Program Files\foobar2000\foobar2000.exe""");
			Console.WriteLine("  Patches the foobar2000 executable (version 0.9.4 or 0.9.4.1 only) to ignore");
			Console.WriteLine("  the checksum of .cfg files, allowing it to load .cfg file generated from XML");
			Console.WriteLine("  by this tool. This operation permanently alters the .exe file, and only needs");
			Console.WriteLine("  to be performed once. The original .exe is backed up as foobar2000.exe.bak");
			Console.WriteLine();
		}

		private static FileInfo GetNonExistantFilename(FileInfo baseFileName)
		{
			if (!baseFileName.Exists)
				return baseFileName; //The base file name itself is fine, doesn't exist yet

			//Make the filename unique by appending a number to it
			int i = 1;
			do
			{
				FileInfo test = new FileInfo(String.Format("{0} ({1}){2}", Path.Combine(baseFileName.DirectoryName, Path.GetFileNameWithoutExtension(baseFileName.Name)), i++, baseFileName.Extension));
				if (!test.Exists) //Found a unique filename
					return test;
			} while (true);
		}
	}
}
