using System;
using System.Collections.Generic;
using System.Text;
using FooCfgToXml;
using System.Xml;
using System.IO;

namespace RunInterpreters
{
	public class ServicesInterpreter : IValueInterpreter
	{
		private static readonly Guid sSettingGuid = new Guid("38c33677-fba4-4f0f-9787-3dff480cb41a");

		public Guid SettingGuid
		{
			get { return sSettingGuid; }
		}

		public void WriteValueToXml(XmlWriter writer, byte[] value)
		{
			MemoryStream reader = new MemoryStream(value, false);
			writer.WriteStartElement("RunServices");

			int numberOfServices = CfgToXmlUtilities.ReadInt(reader);
			for (int i = 0; i < numberOfServices; i++)
			{
				string label = CfgToXmlUtilities.ReadString(reader, false);
				string path = CfgToXmlUtilities.ReadString(reader, false);
				bool minimize = CfgToXmlUtilities.ReadBool(reader);
				byte simultaneousRuns = (byte)reader.ReadByte();
				
				writer.WriteStartElement("Service");
				writer.WriteAttributeString("label", label);
				writer.WriteAttributeString("minimize", XmlConvert.ToString(minimize));
				writer.WriteAttributeString("simultaneousRuns", SimultaneousRunsToString(simultaneousRuns));
				writer.WriteString(path);

				writer.WriteEndElement(); //Service
			}

			writer.WriteEndElement(); //RunServices
		}

		public void WriteXmlToValue(Stream writer, XmlElement value)
		{
			XmlElement runServices = (XmlElement)value.FirstChild; //Should be first and only child

			XmlNodeList serviceElements = runServices.SelectNodes("Service");
			XmlToCfgUtilities.WriteInt(writer, serviceElements.Count);
			foreach (XmlElement serviceElement in serviceElements)
			{
				XmlToCfgUtilities.WriteString(writer, serviceElement.GetAttribute("label"), true, false);
				XmlToCfgUtilities.WriteString(writer, serviceElement.InnerText, true, false);
				XmlToCfgUtilities.WriteBool(writer, XmlConvert.ToBoolean(serviceElement.GetAttribute("minimize")));
				writer.WriteByte(StringToSimultaneousRuns(serviceElement.GetAttribute("simultaneousRuns")));
			}
		}

		private string SimultaneousRunsToString(byte simultaneousRuns)
		{
			if (simultaneousRuns == 50)
				return "unlimited";

			return XmlConvert.ToString(simultaneousRuns);
		}

		private byte StringToSimultaneousRuns(string simultaneousRuns)
		{
			if (simultaneousRuns == "unlimited")
				return 50;

			return XmlConvert.ToByte(simultaneousRuns);
		}
	}
}
